;; Generated package description from seq.el  -*- no-byte-compile: t -*-
(define-package "seq" "2.22.0.20201201.220543" "Sequence manipulation functions" 'nil :url "https://elpa.gnu.org/packages/seq.html" :keywords '("sequences") :authors '(("Nicolas Petton" . "nicolas@petton.fr")) :maintainer '(nil . "emacs-devel@gnu.org"))
